import 'package:appwrite/appwrite.dart';
import 'package:car_app/exception/authentication.dart';
import 'package:car_app/exception/server.dart';
import 'package:car_app/model/user.dart';
import 'package:car_app/service/appwrite/appwrite.dart';
import 'package:car_app/service/interface/authentication.dart';

class AppwriteAuthentication implements AuthenticationService<String> {
  @override
  String get usernameFieldName => 'Email';
  
  @override
  String get logoPath => 'assets/images/appwrite_logo.png';
  
  @override
  String get providerName => 'Appwrite';

  @override
  Future<User<String>> get user => _getUser();

  static const String _currentSessionId = 'current';
  static const String _errorMessageSplitPattern = 'param: ';
  static const String _defaultErrorMessage = 'Something went wrong';

  final Account _account = AppwriteService().account;
  
  @override
  Future<bool> isLoggedIn() async {
    try {
      await _account.get();
      return true;
    } on AppwriteException catch (_)  {
      return false;
    }
  }

  @override
  Future<void> loginUsernamePassword(String username, String password) async {
    await _tryWithException(() async => await _account.createEmailSession(email: username, password: password));
  }

  @override
  Future<void> logout() async {
    await _account.deleteSession(sessionId: _currentSessionId);
  }
  
  @override
  Future<void> register(String name, String email, String password) async {
    await _tryWithException(() => _account.create(userId: ID.unique(), email: email, password: password, name: name));
  }

  Future<void> _tryWithException(Future<void> Function() function) async {
    try {
      await function();
    } on AppwriteException catch (e) {
      var errorMessage = _makeErrorMessage(e.message);
      throw AuthenticationException(message: errorMessage);
    }
  }

  String _makeErrorMessage(String? errorMessage) {
    if (errorMessage == null) {
      return _defaultErrorMessage;
    }
    else if (errorMessage.contains(_errorMessageSplitPattern)) {
      return errorMessage.split(_errorMessageSplitPattern)[1];
    }
    return errorMessage;
  }

  Future<User<String>> _getUser() async {
    try {
      var user = await _account.get();
      return User(
        id: user.$id,
        username: user.email,
        name: user.name,
      );
    } on AppwriteException catch (_) {
      throw ServerException(action: "get user");
    }
  }
}
