import 'package:car_app/exception/authentication.dart';
import 'package:car_app/exception/key_not_found.dart';
import 'package:car_app/model/user.dart';
import 'package:car_app/service/interface/authentication.dart';
import 'package:car_app/service/interface/key_value_storage.dart';

class CachedUsernamePasswordAuthenticationService<T> implements AuthenticationService<T> {
  final AuthenticationService<T> authenticationService;
  final KeyValueStorageService<String, String> storageService;

  late final String usernameKey;
  late final String passwordKey;

  @override
  String get logoPath => authenticationService.logoPath;

  @override
  String get providerName => authenticationService.providerName;

  @override
  String get usernameFieldName => authenticationService.usernameFieldName;

  @override
  Future<User<T>> get user => authenticationService.user;

  CachedUsernamePasswordAuthenticationService({required this.authenticationService, required this.storageService}) {
    var providerName = authenticationService.providerName.toLowerCase();
    usernameKey = "username_$providerName";
    passwordKey = "password_$providerName";
  }

  @override
  Future<bool> isLoggedIn() async {
    if (await authenticationService.isLoggedIn()) {
      return true;
    }
    var (username, password) = await _getUsernamePassword();
    if (username != null && password != null) {
      try {
        await authenticationService.loginUsernamePassword(username, password);
        return true;
      } on AuthenticationException catch (_) {} //ignore AuthenticationException, as it means login not succeeded
    }

    return false;
  }

  @override
  Future<void> loginUsernamePassword(String username, String password) async {
    await authenticationService.loginUsernamePassword(username, password);
    await _updateUsernamePassword(username, password);
  }

  @override
  Future<void> register(String name, String username, String password) async {
    await authenticationService.register(name, username, password);
    await _updateUsernamePassword(username, password);
  }

  @override
  Future<void> logout() async {
    await authenticationService.logout();
  }

  Future<void> _updateUsernamePassword(String username, String password) async {
    var (savedUsername, savedPassword) = await _getUsernamePassword();

    if (savedUsername != username || savedPassword != password) {
      await storageService.add(usernameKey, password);
      await storageService.add(passwordKey, password);
    }
  }

  Future<(String?, String?)> _getUsernamePassword() async {
    try {
      return (await storageService.get(usernameKey), await storageService.get(passwordKey));
    } on KeyNotFoundException catch (_) {
      return (null, null);
    }
  }
}
