import 'package:car_app/model/calendar_event.dart';
import 'package:car_app/view/calendar_event.dart';
import 'package:car_app/view/defaults.dart';
import 'package:car_app/view/routes_titles.dart';
import 'package:car_app/viewmodel/calendar.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:table_calendar/table_calendar.dart';

class CalendarView extends StatefulWidget {

  const CalendarView({super.key});

  @override
  State<StatefulWidget> createState() => _CalendarState();
}

class _CalendarState extends State<CalendarView> {
  DateTime _focusedDay = DateTime.now();
  DateTime? _selectedDay;
  CalendarFormat _calendarFormat = CalendarFormat.month;
  
  @override
  void initState() {
    var viewModel = Provider.of<CalendarViewModel>(context, listen: false);
    viewModel.loadEvents();

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: DefaultAppBar(
        context,
        titleText: CalendarRouting.title,
      ),
      drawer: DefaultDrawer(
        context,
        currentRoute: CalendarRouting.route
      ),
      body: Consumer<CalendarViewModel>(
        builder: (_, viewModel, __) {
          var selectedDayEvents = viewModel.eventsOfDay(_selectedDay ?? _focusedDay);

          return Column(
            children: [
              TableCalendar<CalendarEvent>(
                focusedDay: _focusedDay,
                firstDay: DateTime.utc(1999, 01, 01), // Not possible to set in onPageChanged
                lastDay: DateTime.utc(2100, 12, 31),
                calendarFormat: _calendarFormat,
                startingDayOfWeek: StartingDayOfWeek.monday,
                pageJumpingEnabled: true,
                eventLoader: (day) {
                  return viewModel.eventsOfDay(day);
                },
                headerStyle: const HeaderStyle(
                  formatButtonShowsNext: false,
                ),
                calendarStyle: CalendarStyle(
                  todayDecoration: BoxDecoration(
                    shape: BoxShape.circle,
                    color: Theme.of(context).colorScheme.inversePrimary,
                  ),
                  selectedDecoration: BoxDecoration(
                    shape: BoxShape.circle,
                    color: Theme.of(context).colorScheme.primary,
                  ),
                  markerDecoration: BoxDecoration(
                    shape: BoxShape.circle,
                    color: Theme.of(context).colorScheme.inverseSurface,
                  ),
                  canMarkersOverflow: false,
                ),

                selectedDayPredicate: (day) => isSameDay(_selectedDay, day),
                onDaySelected: (selectedDay, focusedDay) {
                  setState(() {
                    _selectedDay = selectedDay;
                    _focusedDay = focusedDay;
                  });
                },

                onFormatChanged: (calendarFormat) {
                  setState(() {
                    _calendarFormat = calendarFormat;
                  });
                },
              ),
              
              const SizedBox(height: 8),

              Expanded(
                child: Stack(
                  children: [
                    ListView.builder(
                      itemCount: selectedDayEvents.length,
                      itemBuilder: (context, index) {
                        return Card(
                          child: ListTile(
                            leading: CircleAvatar(
                              child: Text(viewModel.avatarTextFromName(selectedDayEvents[index].user.name)),
                            ),
                            title: Text(selectedDayEvents[index].title),
                            subtitle: Text(selectedDayEvents[index].description ?? ""),
                            onTap: () {
                              var eventToUpdate = selectedDayEvents[index];
                              Navigator.push(context, MaterialPageRoute<CalendarEvent>(builder: (context) => CalendarEventView(
                                initialEvent: eventToUpdate,
                              )))
                              .then((event) {
                                if (event != null) {
                                  viewModel.replaceEvent(eventToUpdate, event);
                                }
                              });
                            },
                            onLongPress: () {
                              var event = selectedDayEvents[index];
                              if(viewModel.isEventFromCurrentUser(event)) {
                                viewModel.deleteEvent(event, (CalendarEvent event) {
                                  var messenger = ScaffoldMessenger.of(context);
                                  var undoSnackBar = UndoSnackBar(contentText: "Event deleted", undoAction: () {
                                    viewModel.addEvent(event);
                                    messenger.hideCurrentSnackBar();
                                  });
                                  messenger.showSnackBar(undoSnackBar);
                                });
                              }
                            },
                          ),
                        );
                      },
                    ),
                    Align(
                      alignment: Alignment.bottomRight,
                      child: Container(
                        margin: const EdgeInsets.only(right: 10, bottom: 10),
                        child: ElevatedButton(
                          onPressed: () {
                            Navigator.push(context, MaterialPageRoute<CalendarEvent>(builder: (context) => CalendarEventView(
                              initialStartTime: viewModel.getSelectedDateWithCurrentTime(_selectedDay),
                            ))).then((event) {
                              if (event != null) {
                                viewModel.addNewlyAddedCalendarEvent(event);
                              }
                            });
                          },
                          style: ElevatedButton.styleFrom(
                            shape: const CircleBorder(),
                            padding: const EdgeInsets.all(2),
                          ),
                          child: const Icon(Icons.add)
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            ],
          );
        },
      ),
    );
  }
}
